package com.spring.series.bean.lifecycle;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanFactoryAware;
import org.springframework.beans.factory.BeanNameAware;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

public class Employee implements BeanNameAware, BeanFactoryAware, ApplicationContextAware, InitializingBean, DisposableBean {

	// instance variables
	private String name;
	private int age;

	/**
	 * getter and setter
	 */
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public int getAge() {
		return age;
	}
	public void setAge(int age) {
		this.age = age;
	}

	@Override
	public void setBeanName(String bean) {
		System.out.println("\nBeanNameAware/setBeanName : " + bean);
	}

	@Override
	public void setBeanFactory(BeanFactory beanFactory) throws BeansException {
		System.out.println("\nBeanFactoryAware/setBeanFactory : " + beanFactory);
	}

	@Override
	public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
		System.out.println("\nApplicationContextAware/setApplicationContext : " + applicationContext);
	}

	@PostConstruct
	public void customInit(){
		System.out.println("Methods annotated with @PostConstruct");
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		System.out.println("\nafterPropertiesSet() as defined by the InitializingBean callback interface");
	}

	// declared in bean definition with attribute init-method="initXML"
	public void initXML(){
		System.out.println("\nA custom configured init() method : setting attribute init-method attribute in the bean defintion");
	}

	/**
	 * bean method to printout the details of the employee instance
	 */
	public void printEmployeeDetails() {

		System.out.println("******************************************************************");
		System.out.println("Employee Name \t\t: " + name);
		System.out.println("Employee Age \t\t: " + age);
		System.out.println("******************************************************************");
	}

	@PreDestroy
	public void customDestroy(){
		System.out.println("\nMethods annotated with @PreDestroy");
	}

	@Override
	public void destroy() throws Exception {
		System.out.println("\ndestroy() as defined by the DisposableBean callback interface");
	}

	// declared in bean definition with attribute destroy-method="destroyXML"
	public void destroyXML(){
		System.out.println("\nA custom configured destroy() method : setting attribute destroy-method attribute in the bean defintion");
	}
}